@@ -26,6 +26,7 @@
 #include "pam.h"
 #include "cfg.h"
 #include <sec-mod-auth.h>
+#include <ccan/hash/hash.h>
 
 #ifdef HAVE_PAM
 
@@ -80,13 +81,19 @@ unsigned i;
 				break;
 			case PAM_PROMPT_ECHO_OFF:
 			case PAM_PROMPT_ECHO_ON:
-				syslog(LOG_DEBUG, "PAM-auth conv: echo-%s, sent: %d", (msg[i]->msg_style==PAM_PROMPT_ECHO_ON)?"on":"off", pctx->sent_msg);
-
 				if (pctx->sent_msg == 0) {
 					/* no message, just asking for password */
 					str_reset(&pctx->msg);
 					pctx->sent_msg = 1;
+
 				}
+
+				if (msg[i]->msg) {
+					str_append_str(&pctx->msg, msg[i]->msg);
+				}
+
+				syslog(LOG_DEBUG, "PAM-auth conv: echo-%s, msg: '%s'", (msg[i]->msg_style==PAM_PROMPT_ECHO_ON)?"on":"off", msg[i]->msg!=NULL?msg[i]->msg:"");
+
 				pctx->state = PAM_S_WAIT_FOR_PASS;
 				pctx->cr_ret = PAM_SUCCESS;
 				co_resume();
@@ -141,7 +148,7 @@ wait:
 	}
 }
 
-static int pam_auth_init(void** ctx, void *pool, const char* user, const char* ip)
+static int pam_auth_init(void** ctx, void *pool, const char* user, const char* ip, const char *our_ip, unsigned pid)
 {
 int pret;
 struct pam_ctx_st * pctx;
@@ -186,12 +193,12 @@ fail1:
 	return -1;
 }
 
-static int pam_auth_msg(void* ctx, void *pool, char** msg)
+static int pam_auth_msg(void* ctx, void *pool, passwd_msg_st *pst)
 {
 struct pam_ctx_st * pctx = ctx;
+size_t prompt_hash = 0;
 
 	if (pctx->state != PAM_S_INIT && pctx->state != PAM_S_WAIT_FOR_PASS) {
-		*msg = NULL;
 		return 0;
 	}
 
@@ -206,20 +213,28 @@ struct pam_ctx_st * pctx = ctx;
 		}
 	}
 
-	if (msg != NULL) {
-		if (pctx->msg.length == 0)
-                        if (pctx->changing)
-				*msg = talloc_strdup(pool, "Please enter the new password.");
-                        else
-				*msg = talloc_strdup(pool, "Please enter your password.");
-		else {
-			if (str_append_data(&pctx->msg, "\0", 1) < 0)
-				return -1;
+	if (pctx->msg.length == 0) {
+                if (pctx->changing)
+			pst->msg_str = talloc_strdup(pool, "Please enter the new password.");
+                /* else use the default prompt */
+	} else {
+		if (str_append_data(&pctx->msg, "\0", 1) < 0)
+			return -1;
 
-			*msg = talloc_strdup(pool, (char*)pctx->msg.data);
-		}
+		prompt_hash = hash_any(pctx->msg.data, pctx->msg.length, 0);
+
+		pst->msg_str = talloc_strdup(pool, (char*)pctx->msg.data);
 	}
 
+	pst->counter = pctx->passwd_counter;
+
+	/* differentiate password prompts, if the hash of the prompt
+	 * is different. 
+	 */
+	if (pctx->prev_prompt_hash != prompt_hash)
+		pctx->passwd_counter++;
+	pctx->prev_prompt_hash = prompt_hash;
+
 	return 0;
 }
 
