@@ -81,7 +81,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "tcp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "max-ban-score", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "ban-points-wrong-password", .type = OPTION_NUMERIC, .mandatory = 0 },
-	{ .name = "ban-connection", .type = OPTION_NUMERIC, .mandatory = 0 },
+	{ .name = "ban-points-connection", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "ban-points-kkdcp", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "udp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "keepalive", .type = OPTION_NUMERIC, .mandatory = 0 },
@@ -122,6 +122,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "deny-roaming", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "use-utmp", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "use-dbus", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "persistent-cookies", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "use-occtl", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "try-mtu-discovery", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "ping-leases", .type = OPTION_BOOLEAN, .mandatory = 0 },
@@ -131,6 +132,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "net-priority", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "output-buffer", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "cookie-timeout", .type = OPTION_NUMERIC, .mandatory = 0 },
+	{ .name = "session-timeout", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "stats-report-time", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "rekey-time", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "rekey-method", .type = OPTION_STRING, .mandatory = 0 },
@@ -381,6 +383,9 @@ static void figure_auth_funcs(struct perm_cfg_st *config, char **auth, unsigned
 	unsigned j, i;
 	unsigned found;
 
+	if (auth == NULL)
+		return;
+
 	if (primary != 0) {
 		/* Set the primary authentication methods */
 		for (j=0;j<auth_size;j++) {
@@ -453,7 +458,6 @@ static void figure_auth_funcs(struct perm_cfg_st *config, char **auth, unsigned
 			talloc_free(auth[j]);
 		}
 		config->auth_methods = x;
-
 	}
 	talloc_free(auth);
 }
@@ -777,10 +781,9 @@ unsigned urlfw_size = 0;
 	} else {
 		READ_TF("isolate-workers", config->isolate, 0);
 	}
-#if !defined(HAVE_LIBSECCOMP) && !defined(ENABLE_LINUX_NS)
+#if !defined(HAVE_LIBSECCOMP)
 	if (config->isolate != 0) {
 		fprintf(stderr, "error: 'isolate-workers' is set to true, but not compiled with seccomp or Linux namespaces support\n");
-		exit(1);
 	}
 #endif
 
@@ -840,6 +843,9 @@ unsigned urlfw_size = 0;
 	READ_NUMERIC("cookie-timeout", config->cookie_timeout);
 	if (config->cookie_timeout == 0)
 		config->cookie_timeout = DEFAULT_COOKIE_RECON_TIMEOUT;
+	READ_TF("persistent-cookies", config->persistent_cookies, 0);
+
+	READ_NUMERIC("session-timeout", config->session_timeout);
 
 	READ_NUMERIC("auth-timeout", config->auth_timeout);
 	READ_NUMERIC("idle-timeout", config->idle_timeout);
@@ -1174,9 +1180,6 @@ void print_version(tOptions *opts, tOptDesc *desc)
 #ifdef HAVE_LIBSECCOMP
 	fprintf(stderr, "seccomp, ");
 #endif
-#ifdef ENABLE_LINUX_NS
-	fprintf(stderr, "Linux-NS, ");
-#endif
 #ifdef HAVE_LIBWRAP
 	fprintf(stderr, "tcp-wrappers, ");
 #endif
