@@ -167,18 +167,18 @@ struct proc_st *ctmp;
 
 /* k: whether to kill the process
  */
-void remove_proc(main_server_st * s, struct proc_st *proc, unsigned k)
+void remove_proc(main_server_st * s, struct proc_st *proc, unsigned flags)
 {
 	mslog(s, proc, LOG_INFO, "user disconnected");
 
 	list_del(&proc->list);
 	s->active_clients--;
 
-	if (k && proc->pid != -1 && proc->pid != 0)
+	if ((flags&RPROC_KILL) && proc->pid != -1 && proc->pid != 0)
 		kill(proc->pid, SIGTERM);
 
 	/* close any pending sessions */
-	if (proc->active_sid) {
+	if (proc->active_sid && !(flags & RPROC_QUIT)) {
 		session_close(s, proc);
 	}
 
@@ -313,7 +313,9 @@ int handle_commands(main_server_st * s, struct proc_st *proc)
 	hdr.msg_iov = iov;
 	hdr.msg_iovlen = 2;
 
-	ret = recvmsg(proc->fd, &hdr, 0);
+	do {
+		ret = recvmsg(proc->fd, &hdr, 0);
+	} while (ret == -1 && errno == EINTR);
 	if (ret == -1) {
 		e = errno;
 		mslog(s, proc, LOG_ERR,
@@ -593,11 +595,14 @@ int handle_commands(main_server_st * s, struct proc_st *proc)
 	return ret;
 }
 
-/* Returns a file descriptor to be used for communication with sec-mod
+/* Returns two file descriptors to be used for communication with sec-mod.
+ * The sync_fd is used by main to send synchronous commands- commands which
+ * expect a reply immediately.
  */
-int run_sec_mod(main_server_st * s)
+int run_sec_mod(main_server_st * s, int *sync_fd)
 {
 	int e, fd[2], ret;
+	int sfd[2];
 	pid_t pid;
 	const char *p;
 
@@ -619,6 +624,12 @@ int run_sec_mod(main_server_st * s)
 		exit(1);
 	}
 
+	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, sfd);
+	if (ret < 0) {
+		mslog(s, NULL, LOG_ERR, "error creating sec-mod sync command socket");
+		exit(1);
+	}
+
 	pid = fork();
 	if (pid == 0) {		/* child */
 		clear_lists(s);
@@ -631,11 +642,17 @@ int run_sec_mod(main_server_st * s)
 #endif
 		setproctitle(PACKAGE_NAME "-secmod");
 		close(fd[1]);
-		sec_mod_server(s->main_pool, s->perm_config, p, s->cookie_key, fd[0]);
+		close(sfd[1]);
+		set_cloexec_flag (fd[0], 1);
+		set_cloexec_flag (sfd[0], 1);
+		sec_mod_server(s->main_pool, s->perm_config, p, s->cookie_key, fd[0], sfd[0]);
 		exit(0);
 	} else if (pid > 0) {	/* parent */
 		close(fd[0]);
 		s->sec_mod_pid = pid;
+		set_cloexec_flag (fd[1], 1);
+		set_cloexec_flag (sfd[1], 1);
+		*sync_fd = sfd[1];
 		return fd[1];
 	} else {
 		e = errno;
