@@ -45,6 +45,8 @@
 # define _ATTR_PACKED
 #endif
 
+#define MAX_MSG_SIZE 16*1024
+
 typedef enum {
 	SOCK_TYPE_TCP,
 	SOCK_TYPE_UDP,
@@ -70,6 +72,9 @@ typedef enum {
 #define MIN_NO_COMPRESS_LIMIT 64
 #define DEFAULT_NO_COMPRESS_LIMIT 256
 
+/* Timeout (secs) for communication between main and sec-mod */
+#define MAIN_SEC_MOD_TIMEOUT 120
+
 #define DEBUG_BASIC 1
 #define DEBUG_HTTP  4
 #define DEBUG_TRANSFERRED 5
@@ -104,6 +109,15 @@ extern int syslog_open;
 #define ACCT_TYPE_PAM (1<<1)
 #define ACCT_TYPE_RADIUS (1<<2)
 
+/* User Disconnect reasons (must be > 0) */
+#define REASON_ANY 1
+#define REASON_USER_DISCONNECT 2
+#define REASON_SERVER_DISCONNECT 3
+#define REASON_IDLE_TIMEOUT 4
+#define REASON_DPD_TIMEOUT 5
+#define REASON_ERROR 6
+#define REASON_SESSION_TIMEOUT 7
+
 #define ERR_SUCCESS 0
 #define ERR_BAD_COMMAND -2
 #define ERR_AUTH_FAIL -3
@@ -155,12 +169,15 @@ typedef enum {
 	SM_CMD_CLI_STATS,
 
 	/* from main to sec-mod and vice versa */
-	SM_CMD_AUTH_SESSION_OPEN=240,
-	SM_CMD_AUTH_SESSION_CLOSE,
+	MIN_SM_MAIN_CMD=239,
+	SM_CMD_AUTH_SESSION_OPEN, /* sync: reply is SM_CMD_AUTH_SESSION_REPLY */
+	SM_CMD_AUTH_SESSION_CLOSE, /* sync: reply is SM_CMD_AUTH_CLI_STATS */
 	SM_CMD_AUTH_SESSION_REPLY,
 	SM_CMD_AUTH_BAN_IP,
 	SM_CMD_AUTH_BAN_IP_REPLY,
 	SM_CMD_AUTH_CLI_STATS,
+
+	MAX_SM_MAIN_CMD,
 } cmd_request_t;
 
 struct group_cfg_st {
@@ -197,6 +214,11 @@ struct group_cfg_st {
 	size_t rx_per_sec;
 	size_t tx_per_sec;
 
+	/* the number of secs to send interim updates. If set, it overrides
+	 * stats-report-time. */
+	unsigned interim_update_secs;
+	unsigned session_timeout_secs;
+
 	unsigned deny_roaming; /* whether the user is allowed to re-use cookies from another IP */
 	unsigned net_priority;
 	unsigned no_udp; /* whether to disable UDP for this user */
@@ -308,6 +330,8 @@ struct cfg_st {
 
 	unsigned deny_roaming; /* whether a cookie is restricted to a single IP */
 	time_t cookie_timeout;	/* in seconds */
+	time_t session_timeout;	/* in seconds */
+	unsigned persistent_cookies; /* whether cookies stay valid after disconnect */
 
 	time_t rekey_time;	/* in seconds */
 	unsigned rekey_method; /* REKEY_METHOD_ */
