@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013, 2014 Nikos Mavrogiannopoulos
+ * Copyright (C) 2013, 2014, 2015 Nikos Mavrogiannopoulos
+ * Copyright (C) 2015 Red Hat, Inc.
  *
  * This file is part of ocserv.
  *
@@ -74,8 +75,11 @@ static const char oc_login_msg_end[] =
 static const char login_msg_user[] =
     "<input type=\"text\" name=\"username\" label=\"Username:\" />\n";
 
-static const char login_msg_password[] =
-    "<input type=\"password\" name=\"password\" label=\"Password:\" />\n";
+#define DEFAULT_PASSWD_LABEL "Password:"
+#define LOGIN_MSG_PASSWORD \
+    "<input type=\"password\" name=\"password\" label=\""DEFAULT_PASSWD_LABEL"\" />\n"
+#define LOGIN_MSG_PASSWORD_CTR \
+    "<input type=\"password\" name=\"password\" label=\"Password%d:\" />\n"
 
 #define OCV3_LOGIN_MSG_START \
     "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" \
@@ -162,7 +166,7 @@ static int append_group_str(worker_st * ws, str_st *str, const char *group)
 	return 0;
 }
 
-int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg)
+int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg, unsigned pcounter)
 {
 	int ret;
 	char context[BASE64_LENGTH(SID_SIZE) + 1];
@@ -223,8 +227,8 @@ int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg)
 
 	if (ws->auth_state == S_AUTH_REQ) {
 		/* only ask password */
-		if (pmsg == NULL)
-			pmsg = "Please enter your password";
+		if (pmsg == NULL || strncasecmp(pmsg, DEFAULT_PASSWD_LABEL, sizeof(DEFAULT_PASSWD_LABEL)-1) == 0)
+			pmsg = "Please enter your password.";
 
 		ret = str_append_printf(&str, login_msg_start, pmsg);
 		if (ret < 0) {
@@ -232,7 +236,10 @@ int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg)
 			goto cleanup;
 		}
 
-		ret = str_append_str(&str, login_msg_password);
+		if (pcounter > 0)
+			ret = str_append_printf(&str, LOGIN_MSG_PASSWORD_CTR, pcounter);
+		else
+			ret = str_append_str(&str, LOGIN_MSG_PASSWORD);
 		if (ret < 0) {
 			ret = -1;
 			goto cleanup;
@@ -246,7 +253,7 @@ int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg)
 
 	} else {
 		if (pmsg == NULL)
-			pmsg = "Please enter your username";
+			pmsg = "Please enter your username.";
 
 		/* ask for username and groups */
 		ret = str_append_printf(&str, login_msg_start, pmsg);
@@ -396,7 +403,7 @@ int get_auth_handler2(worker_st * ws, unsigned http_ver, const char *pmsg)
 
 int get_auth_handler(worker_st * ws, unsigned http_ver)
 {
-	return get_auth_handler2(ws, http_ver, NULL);
+	return get_auth_handler2(ws, http_ver, NULL, 0);
 }
 
 int get_cert_names(worker_st * ws, const gnutls_datum_t * raw)
@@ -511,7 +518,8 @@ static int recv_cookie_auth_reply(worker_st * ws)
 
 	ret = recv_socket_msg(ws, ws->cmd_fd, AUTH_COOKIE_REP, &socketfd,
 			      (void *)&msg,
-			      (unpack_func) auth_reply_msg__unpack);
+			      (unpack_func) auth_reply_msg__unpack,
+			      DEFAULT_SOCKET_TIMEOUT);
 	if (ret < 0) {
 		oclog(ws, LOG_ERR, "error receiving auth reply message");
 		return ret;
@@ -546,6 +554,18 @@ static int recv_cookie_auth_reply(worker_st * ws)
 			memcpy(ws->session_id, msg->session_id.data,
 			       msg->session_id.len);
 
+			if (msg->has_interim_update_secs) {
+				oclog(ws, LOG_DEBUG, "overriding stats-report-time with auth server's value (%u)",
+				      (unsigned)msg->interim_update_secs);
+				ws->config->stats_report_time = msg->interim_update_secs;
+			}
+
+			if (msg->has_session_timeout_secs) {
+				oclog(ws, LOG_DEBUG, "overriding session-timeout with auth server's value (%u)",
+				      (unsigned)msg->session_timeout_secs);
+				ws->config->session_timeout = msg->session_timeout_secs;
+			}
+
 			if (msg->ipv4 != NULL) {
 				talloc_free(ws->vinfo.ipv4);
 				if (strcmp(msg->ipv4, "0.0.0.0") == 0)
@@ -718,14 +738,15 @@ int connect_to_secmod(worker_st * ws)
 	return sd;
 }
 
-static int recv_auth_reply(worker_st * ws, int sd, char **txt)
+static int recv_auth_reply(worker_st * ws, int sd, char **txt, unsigned *pcounter)
 {
 	int ret;
 	SecAuthReplyMsg *msg = NULL;
 	PROTOBUF_ALLOCATOR(pa, ws);
 
 	ret = recv_msg(ws, sd, SM_CMD_AUTH_REP,
-		       (void *)&msg, (unpack_func) sec_auth_reply_msg__unpack);
+		       (void *)&msg, (unpack_func) sec_auth_reply_msg__unpack,
+		       DEFAULT_SOCKET_TIMEOUT);
 	if (ret < 0) {
 		oclog(ws, LOG_ERR, "error receiving auth reply message");
 		return ret;
@@ -738,12 +759,16 @@ static int recv_auth_reply(worker_st * ws, int sd, char **txt)
 
 	switch (msg->reply) {
 	case AUTH__REP__MSG:
-		if (txt == NULL || msg->msg == NULL) {
-			oclog(ws, LOG_ERR, "received unexpected msg");
-			return ERR_AUTH_FAIL;
-		}
+		if (msg->msg)
+			*txt = talloc_strdup(ws, msg->msg);
+		else
+			*txt = NULL;
+
+		if (msg->has_passwd_counter)
+			*pcounter = msg->passwd_counter;
+		else
+			*pcounter = 0;
 
-		*txt = talloc_strdup(ws, msg->msg);
 		if (msg->has_sid && msg->sid.len == sizeof(ws->sid)) {
 			/* update our sid */
 			memcpy(ws->sid, msg->sid.data, sizeof(ws->sid));
@@ -1008,10 +1033,100 @@ int post_common_handler(worker_st * ws, unsigned http_ver, const char *imsg)
 	return 0;
 }
 
-#define XMLUSER "<username>"
-#define XMLPASS "<password>"
-#define XMLUSER_END "</username>"
-#define XMLPASS_END "</password>"
+/* Returns the contents of the password field in a newly allocated
+ * string, or a negative value on error.
+ *
+ * @body: is the string to search the xml field at, should be null-terminated.
+ * @value: the value that was found
+ */
+static
+int match_password_in_reply(worker_st * ws, char *body, unsigned body_length,
+			    char **value)
+{
+	char *p;
+	unsigned len, xml = 0;
+
+	if (body == NULL || body_length == 0)
+		return -1;
+
+	if (memmem(body, body_length, "<?xml", 5) != 0) {
+		xml = 1;
+
+		/* body should contain <password?>test</password?> */
+		*value =
+		    strcasestr(body, "<password");
+		if (*value == NULL) {
+			oclog(ws, LOG_HTTP_DEBUG,
+			      "cannot find password in client XML message");
+			return -1;
+		}
+		/* find terminator */
+		p = strchr(*value, '>');
+		if (p == NULL) {
+			oclog(ws, LOG_HTTP_DEBUG,
+			      "unterminated password in client XML message");
+			return -1;
+		}
+		p++;
+
+		*value = p;
+		len = 0;
+		while (*p != 0) {
+			if (*p == '<'
+			    && (strncasecmp(p, "</password", sizeof("</password")-1) == 0)) {
+				break;
+			}
+			p++;
+			len++;
+		}
+	} else {		/* non-xml version */
+		/* body should be "username=test&password?=test" */
+		*value =
+		    strcasestr(body, "password");
+		if (*value == NULL) {
+			oclog(ws, LOG_HTTP_DEBUG,
+			      "cannot find password in client message");
+			return -1;
+		}
+
+		p = strchr(*value, '=');
+		if (p == NULL) {
+			oclog(ws, LOG_HTTP_DEBUG,
+			      "unterminated password in client message");
+			return -1;
+		}
+		p++;
+
+		*value = p;
+		len = 0;
+		while (*p != 0) {
+			if (*p == '&') {
+				break;
+			}
+			p++;
+			len++;
+		}
+	}
+
+	if (len == 0) {
+		*value = talloc_strdup(ws->req.body, "");
+		if (*value != NULL)
+			return 0;
+		return -1;
+	}
+	if (xml)
+		*value = unescape_html(ws->req.body, *value, len, NULL);
+	else
+		*value = unescape_url(ws->req.body, *value, len, NULL);
+
+	if (*value == NULL) {
+		oclog(ws, LOG_ERR,
+		      "password requested but no such field in client message");
+		return -1;
+	}
+
+	return 0;
+}
 
 /* Returns the contents of the provided fields in a newly allocated
  * string, or a negative value on error.
@@ -1166,8 +1281,27 @@ int basic_auth_handler(worker_st * ws, unsigned http_ver, const char *msg)
 	return ret;
 }
 
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
+# define gsocklen int
+#else
+# define gsocklen socklen_t
+#endif
+
+static char *get_our_ip(int fd, char str[MAX_IP_STR])
+{
+	int ret;
+	struct sockaddr_storage sockaddr;
+	gsocklen socklen;
+
+	socklen = sizeof(sockaddr);
+	ret = getsockname(fd, (struct sockaddr*)&sockaddr, &socklen);
+	if (ret == -1)
+		return NULL;
+
+	return human_addr2((struct sockaddr*)&sockaddr, socklen, str, MAX_IP_STR, 0);
+}
+
 #define USERNAME_FIELD "username"
-#define PASSWORD_FIELD "password"
 #define GROUPNAME_FIELD "group%5flist"
 #define GROUPNAME_FIELD2 "group_list"
 #define GROUPNAME_FIELD_XML "group-select"
@@ -1185,8 +1319,10 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 	char *username = NULL;
 	char *password = NULL;
 	char *groupname = NULL;
+	char our_ip_str[MAX_IP_STR];
 	char *msg = NULL;
 	unsigned def_group = 0;
+	unsigned pcounter = 0;
 
 	if (req->body_length > 0) {
 		oclog(ws, LOG_HTTP_DEBUG, "POST body: '%.*s'", (int)req->body_length,
@@ -1271,7 +1407,7 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 
 			if (def_group == 0 && ws->cert_groups_size > 0 && ws->groupname[0] == 0) {
 				oclog(ws, LOG_HTTP_DEBUG, "user has not selected a group");
-				return get_auth_handler2(ws, http_ver, "Please select your group");
+				return get_auth_handler2(ws, http_ver, "Please select your group.", 0);
 			}
 
 			ireq.tls_auth_ok = ws->cert_auth_ok;
@@ -1283,6 +1419,7 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 
 		ireq.hostname = req->hostname;
 		ireq.ip = ws->remote_ip_str;
+		ireq.our_ip = get_our_ip(ws->conn_fd, our_ip_str);
 
 		sd = connect_to_secmod(ws);
 		if (sd == -1) {
@@ -1319,10 +1456,8 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 		}
 
 		if (areq.password == NULL && ws->selected_auth->type & AUTH_TYPE_USERNAME_PASS) {
-			ret = parse_reply(ws, req->body, req->body_length,
-					PASSWORD_FIELD, sizeof(PASSWORD_FIELD)-1,
-					NULL, 0,
-					&password);
+			ret = match_password_in_reply(ws, req->body, req->body_length,
+						      &password);
 			if (ret < 0) {
 				reason = MSG_NO_PASSWORD_ERROR;
 				oclog(ws, LOG_ERR, "failed reading password");
@@ -1370,7 +1505,7 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 		goto auth_fail;
 	}
 
-	ret = recv_auth_reply(ws, sd, &msg);
+	ret = recv_auth_reply(ws, sd, &msg, &pcounter);
 	if (sd != -1) {
 		close(sd);
 		sd = -1;
@@ -1384,7 +1519,7 @@ int post_auth_handler(worker_st * ws, unsigned http_ver)
 		if (ws->selected_auth->type & AUTH_TYPE_GSSAPI) {
 			ret = basic_auth_handler(ws, http_ver, msg);
 		} else {
-			ret = get_auth_handler2(ws, http_ver, msg);
+			ret = get_auth_handler2(ws, http_ver, msg, pcounter);
 		}
 		goto cleanup;
 	} else if (ret < 0) {
